@@ -31,6 +31,7 @@
 #define HEIGHT2 (mfxU16)1440
 
 MFXVideoFrameInterpolation::MFXVideoFrameInterpolation() :
+    m_sequenceEnd(false),
     m_inputFwd(),
     m_inputBkwd(),
     m_enableScd(false),
@@ -48,7 +49,10 @@ MFXVideoFrameInterpolation::MFXVideoFrameInterpolation() :
     m_rgbSurfForFiIn(),
     m_rgbSurfArray(),
     m_outSurfForFi(),
-    m_fiOut()
+    m_fiOut(),
+    m_taskIndex(0),
+    m_time_stamp_start(0),
+    m_time_stamp_interval(0)
 {
 }
 
@@ -106,6 +110,16 @@ mfxStatus MFXVideoFrameInterpolation::ConfigureFrameRate(
     m_outStamp = 0;
     m_outTick = (mfxU16)m_ratio;
 
+    m_time_stamp_start = (mfxU64)MFX_TIMESTAMP_UNKNOWN;
+    // Default to 30fps
+    m_time_stamp_interval = (mfxU64)((mfxF64) MFX_TIME_STAMP_FREQUENCY / (mfxF64)30);
+
+    if (m_frcRational[VPP_OUT].FrameRateExtN != 0)
+    {
+        // Specify the frame rate: FrameRateExtN / FrameRateExtD.
+        m_time_stamp_interval = (mfxU64)(MFX_TIME_STAMP_FREQUENCY * (((mfxF64)m_frcRational[VPP_OUT].FrameRateExtD / (mfxF64)m_frcRational[VPP_OUT].FrameRateExtN)));
+    }
+
     return MFX_ERR_NONE;
 }
 
@@ -116,10 +130,14 @@ mfxStatus MFXVideoFrameInterpolation::InitFrameInterpolator(VideoCORE* core, con
     MFX_CHECK(pD3d11, MFX_ERR_NULL_PTR);
     // Init
     xeAIVfiConfig config = {
-        outInfo.Width, outInfo.Height,
+        outInfo.Width,
+        outInfo.Height,
         (mfxU32)core->GetHWType(),
         pD3d11->GetD3D11Device(),
-        pD3d11->GetD3D11DeviceContext(), DXGI_FORMAT_R8G8B8A8_UNORM };
+        pD3d11->GetD3D11DeviceContext(),
+        DXGI_FORMAT_R8G8B8A8_UNORM,
+        core->GetHWDeviceId()
+        };
     xeAIVfiStatus xeSts = m_aiIntp.Init(config);
     if (xeSts != XE_AIVFI_SUCCESS)
         MFX_RETURN(MFX_ERR_UNKNOWN);
@@ -211,12 +229,20 @@ mfxStatus MFXVideoFrameInterpolation::InitVppAndAllocateSurface(
     const mfxFrameInfo& outInfo,
     const mfxVideoSignalInfo& videoSignalInfo)
 {
+    std::vector<mfxExtBuffer*> extBufferPre, extBufferPost;
+
+    mfxExtVPPScaling m_scalingConfig = {};
+    m_scalingConfig.Header.BufferId = MFX_EXTBUFF_VPP_SCALING;
+    m_scalingConfig.Header.BufferSz = sizeof(mfxExtVPPScaling);
+    m_scalingConfig.ScalingMode = MFX_SCALING_MODE_INTEL_GEN_COMPUTE;
+    //m_scalingConfig.InterpolationMethod = MFX_INTERPOLATION_NEAREST_NEIGHBOR;
+    extBufferPre.push_back(&m_scalingConfig.Header);
+    extBufferPost.push_back(&m_scalingConfig.Header);
+
     mfxExtVideoSignalInfo vsInPre = {};
     mfxExtVideoSignalInfo vsOutPre = {};
     mfxExtVideoSignalInfo vsInPost = {};
     mfxExtVideoSignalInfo vsOutPost = {};
-    mfxExtBuffer* extBufferPre[2] = {};
-    mfxExtBuffer* extBufferPost[2] = {};
     if (videoSignalInfo.enabled)
     {
         mfxVideoSignalInfo vsInfoIn = videoSignalInfo;
@@ -241,16 +267,16 @@ mfxStatus MFXVideoFrameInterpolation::InitVppAndAllocateSurface(
         vsOutPre.VideoFormat = vsInfoOut.VideoFormat;
         vsOutPre.VideoFullRange = vsInfoOut.VideoFullRange;
 
-        extBufferPre[0] = &vsInPre.Header;
-        extBufferPre[1] = &vsOutPre.Header;
+        extBufferPre.push_back(&vsInPre.Header);
+        extBufferPre.push_back(&vsOutPre.Header);
 
         vsInPost = vsOutPre;
         vsInPost.Header.BufferId = MFX_EXTBUFF_VIDEO_SIGNAL_INFO_IN;
         vsOutPost = vsInPre;
         vsOutPost.Header.BufferId = MFX_EXTBUFF_VIDEO_SIGNAL_INFO_OUT;
 
-        extBufferPost[0] = &vsInPost.Header;
-        extBufferPost[1] = &vsOutPost.Header;
+        extBufferPost.push_back(&vsInPost.Header);
+        extBufferPost.push_back(&vsOutPost.Header);
     }
 
     m_vppForFi = IsVppNeededForVfi(inInfo, outInfo);
@@ -276,11 +302,8 @@ mfxStatus MFXVideoFrameInterpolation::InitVppAndAllocateSurface(
             vppParams.vpp.Out.FourCC    = MFX_FOURCC_BGR4;
             vppParams.vpp.Out.PicStruct = inInfo.PicStruct;
 
-            if (videoSignalInfo.enabled)
-            {
-                vppParams.NumExtParam = 2;
-                vppParams.ExtParam = &(extBufferPre[0]);
-            }
+            vppParams.NumExtParam = (mfxU16)extBufferPre.size();
+            vppParams.ExtParam = extBufferPre.data();
 
             sts = m_vppBeforeFi0->Init(&vppParams);
             MFX_CHECK_STS(sts);
@@ -318,11 +341,8 @@ mfxStatus MFXVideoFrameInterpolation::InitVppAndAllocateSurface(
             vppParams.vpp.In.FourCC    = MFX_FOURCC_BGR4;
             vppParams.vpp.Out          = outInfo;
 
-            if (videoSignalInfo.enabled)
-            {
-                vppParams.NumExtParam = 2;
-                vppParams.ExtParam = &(extBufferPost[0]);
-            }
+            vppParams.NumExtParam = (mfxU16)extBufferPost.size();
+            vppParams.ExtParam = extBufferPost.data();
 
             sts = m_vppAfterFi->Init(&vppParams);
             MFX_CHECK_STS(sts);
@@ -366,40 +386,58 @@ mfxStatus MFXVideoFrameInterpolation::UpdateTsAndGetStatus(
     mfxFrameSurface1* output,
     mfxStatus* intSts)
 {
-    if (nullptr == input) return MFX_ERR_MORE_DATA;
     mfxStatus sts = MFX_ERR_NONE;
 
-    if (m_outStamp == 0)
+    if (nullptr == input)
     {
-        m_inputBkwd.Info = output->Info;
-    }
-    else if (m_outStamp == 1)
-    {
-        m_inputFwd.Info = output->Info;
-
-        *intSts = MFX_ERR_MORE_SURFACE;
+        // nullptr == input means input sequence reaches its end
+        if (m_sequenceEnd)
+        {
+            return MFX_ERR_MORE_DATA;
+        }
+        else
+        {
+            if (m_outStamp == (m_ratio - 1)) m_sequenceEnd = true;
+            sts = MFX_ERR_NONE;
+        }
     }
     else
     {
-        *intSts = MFX_ERR_MORE_SURFACE;
+        if (m_outStamp == 0)
+        {
+            // record the time stamp of input frame [n, n+1]
+            m_time_stamp_start = input->Data.TimeStamp;
+            m_inputBkwd.Info = output->Info;
+        }
+        else if (m_outStamp == 1)
+        {
+            m_inputFwd.Info = output->Info;
+
+            *intSts = MFX_ERR_MORE_SURFACE;
+        }
+        else
+        {
+            *intSts = MFX_ERR_MORE_SURFACE;
+        }
     }
+    output->Data.TimeStamp = m_time_stamp_start + m_outStamp * m_time_stamp_interval;
+
+    AddTaskQueue(m_sequenceEnd);
 
     MFX_RETURN(sts);
 }
 
-mfxStatus MFXVideoFrameInterpolation::ReturnSurface(mfxU32 taskIndex, mfxFrameSurface1* out, mfxMemId internalVidMemId)
+mfxStatus MFXVideoFrameInterpolation::ReturnSurface(mfxFrameSurface1* out, mfxMemId internalVidMemId)
 {
     mfxStatus sts = MFX_ERR_NONE;
 
-    while (taskIndex != m_taskQueue.front().first)
-    {
-    }
-
     mfxU32 scdDecision = 0;
 
-    task t = m_taskQueue.front();
+    Task t;
+    m_taskQueue.Dequeue(t);
     //mfxU16 stamp = m_outStamp;
-    mfxU16 stamp = t.second;
+    mfxU16 stamp = t.outStamp;
+    bool isSequenceEnd = t.isSequenceEnd;
     if (stamp == 0)
     {
         if (m_IOPattern & MFX_IOPATTERN_OUT_SYSTEM_MEMORY)
@@ -441,7 +479,7 @@ mfxStatus MFXVideoFrameInterpolation::ReturnSurface(mfxU32 taskIndex, mfxFrameSu
             }
         }
     }
-    else if (stamp == 1)
+    else if (stamp == 1 && !isSequenceEnd)
     {
         if (m_IOPattern & MFX_IOPATTERN_OUT_SYSTEM_MEMORY)
         {
@@ -542,7 +580,7 @@ mfxStatus MFXVideoFrameInterpolation::ReturnSurface(mfxU32 taskIndex, mfxFrameSu
             }
         }
     }
-    else if (stamp >= 2)
+    else if (stamp >= 2 && !isSequenceEnd)
     {
         if (m_IOPattern & MFX_IOPATTERN_OUT_SYSTEM_MEMORY)
         {
@@ -573,17 +611,35 @@ mfxStatus MFXVideoFrameInterpolation::ReturnSurface(mfxU32 taskIndex, mfxFrameSu
         }
         MFX_CHECK_STS(sts);
     }
+    else if (isSequenceEnd)
+    {
+        if (m_IOPattern & MFX_IOPATTERN_OUT_SYSTEM_MEMORY)
+        {
+            sts = m_core->DoFastCopyWrapper(out, MFX_MEMTYPE_SYSTEM_MEMORY, &m_rgbSurfArray[m_ratio], MFX_MEMTYPE_DXVA2_DECODER_TARGET);
+            MFX_CHECK_STS(sts);
+        }
+        else
+        {
+            MFX_CHECK(!internalVidMemId, MFX_ERR_UNDEFINED_BEHAVIOR);
+            if (m_vppForFi)
+            {
+                sts = m_vppAfterFi->Submit(&m_rgbSurfArray[m_ratio], &m_fiOut);
+                MFX_CHECK_STS(sts);
+                sts = m_core->DoFastCopyWrapper(
+                    out, MFX_MEMTYPE_DXVA2_DECODER_TARGET,
+                    &m_fiOut, MFX_MEMTYPE_INTERNAL_FRAME | MFX_MEMTYPE_DXVA2_DECODER_TARGET);
+                MFX_CHECK_STS(sts);
+            }
+            else
+            {
+                sts = m_core->DoFastCopyWrapper(
+                    out, MFX_MEMTYPE_DXVA2_DECODER_TARGET,
+                    &m_rgbSurfArray[m_ratio], MFX_MEMTYPE_INTERNAL_FRAME | MFX_MEMTYPE_DXVA2_DECODER_TARGET);
+                MFX_CHECK_STS(sts);
+            }
+        }
+    }
 
-    //stamp++;
-    //if (stamp == m_outTick)
-    //{
-    //    m_outStamp = 0;
-    //    m_inputBkwd.Data.MemId = m_inputFwd.Data.MemId;
-    //    std::swap(m_memIdBkwd, m_memIdFwd);
-    //    m_inputFwd = {};
-    //    memset(m_output, 0, sizeof(m_output));
-    //}
-    m_taskQueue.pop();
     MFX_RETURN(sts);
 }
 
@@ -725,13 +781,27 @@ mfxStatus MFXVideoFrameInterpolation::SceneChangeDetect(mfxFrameSurface1* input,
     return sts;
 }
 
-mfxStatus MFXVideoFrameInterpolation::AddTaskQueue(mfxU32 taskIndex)
+mfxStatus MFXVideoFrameInterpolation::AddTaskQueue(bool isSequenceEnd)
 {
-    m_taskQueue.push(task(taskIndex, m_outStamp));
-    m_outStamp++;
+    m_taskQueue.Enqueue(Task{ m_taskIndex++, m_outStamp++, isSequenceEnd });
+
     if (m_outStamp == m_outTick)
     {
         m_outStamp = 0;
     }
     return MFX_ERR_NONE;
+}
+
+mfxStatus MFXVideoFrameInterpolation::Query(VideoCORE* core)
+{
+    MFX_CHECK_NULL_PTR1(core);
+    auto platform = core->GetHWType();
+    if (platform == MFX_HW_DG2 || platform >= MFX_HW_MTL)
+    {
+        return MFX_ERR_NONE;
+    }
+    else
+    {
+        MFX_RETURN(MFX_ERR_UNSUPPORTED);
+    }
 }
\ No newline at end of file
