@@ -43,9 +43,7 @@ namespace UMC_AV1_DECODER
         , sequence_header(nullptr)
         , old_seqHdr(nullptr)
         , counter(0)
-        , Curr(nullptr)
-        , Curr_temp(nullptr)
-        , Repeat_show(0)
+        , lastest_submitted_frame(nullptr)
         , PreFrame_id(0)
         , OldPreFrame_id(0)
         , frame_order(0)
@@ -68,6 +66,7 @@ namespace UMC_AV1_DECODER
     {
         outputed_frames.clear();
         m_prev_frame_header = {};
+        last_frame_header   = {};
     }
 
     AV1Decoder::~AV1Decoder()
@@ -169,6 +168,19 @@ namespace UMC_AV1_DECODER
         return false;
     }
 
+
+    void AV1Decoder::AV1IncrementReference(const std::string& function, int line, AV1DecoderFrame* frame)
+    {
+        frame->IncrementReference();
+        DPBLOG_PRINT(function, line, "[+]", frame, frame->GetRefCounter());
+    }
+
+    void AV1Decoder::AV1DecrementReference(const std::string& function, int line, AV1DecoderFrame* frame)
+    {
+        frame->DecrementReference();
+        DPBLOG_PRINT(function, line, "[-]", frame, frame->GetRefCounter());
+    }
+
     UMC::Status AV1Decoder::DecodeHeader(UMC::MediaData* in, UMC_AV1_DECODER::AV1DecoderParams& par)
     {
         if (!in)
@@ -261,7 +273,7 @@ namespace UMC_AV1_DECODER
         return UMC::UMC_OK;
     }
 
-    DPBType AV1Decoder::DPBUpdate(AV1DecoderFrame * prevFrame)
+    DPBType AV1Decoder::ReferenceListUpdate(AV1DecoderFrame * prevFrame)
     {
         assert(prevFrame);
 
@@ -276,7 +288,7 @@ namespace UMC_AV1_DECODER
 
         const FrameHeader& fh = prevFrame->GetFrameHeader();
 
-        prevFrame->DpbUpdated(true);
+        prevFrame->RefUpdated(true);
 
         if (fh.refresh_frame_flags == 0)
             return updatedFrameDPB;
@@ -286,10 +298,12 @@ namespace UMC_AV1_DECODER
             if ((fh.refresh_frame_flags >> i) & 1)
             {
                 if (!prevFrameDPB.empty() && prevFrameDPB[i] && prevFrameDPB[i]->UID != -1)
-                    prevFrameDPB[i]->DecrementReference();
+                {
+                    AV1DecrementReference(__FUNCTION__, __LINE__, prevFrameDPB[i]);
+                }
 
                 updatedFrameDPB[i] = const_cast<AV1DecoderFrame*>(prevFrame);
-                prevFrame->IncrementReference();
+                AV1IncrementReference(__FUNCTION__, __LINE__, prevFrame); 
             }
         }
 
@@ -395,30 +409,20 @@ namespace UMC_AV1_DECODER
         if (fh.show_existing_frame)
         {
             std::unique_lock<std::mutex> l(guard);
+            
+            //get repeat frame
             pFrame = frameDPB[fh.frame_to_show_map_idx];
-
-            if (pFrame->Repeated())
-                return nullptr;
-
-            //Increase referernce here, and will be decreased when
-            //CompleteDecodedFrames not show_frame case.
-            pFrame->IncrementReference();
             assert(pFrame);
             repeateFrame = pFrame->GetMemID();
 
-            //Add one more Reference, and add it into outputed frame list
-            //When QueryFrame finished and update status in outputed frame
-            //list, then it will be released in CompleteDecodedFrames.
-            pFrame->IncrementReference();
-            pFrame->Repeated(true);
-            outputed_frames.push_back(pFrame);
-
-            FrameHeader const& refFH = pFrame->GetFrameHeader();
-
-            if (!refFH.showable_frame)
-                throw av1_exception(UMC::UMC_ERR_INVALID_STREAM);
+            //repeat frame reference counter increase here, and will decrease in queryframe()
+            AV1IncrementReference(__FUNCTION__, __LINE__, pFrame);
 
             FrameHeader const& Repeat_H = pFrame->GetFrameHeader();
+            if (!Repeat_H.showable_frame)
+                throw av1_exception(UMC::UMC_ERR_INVALID_STREAM);
+
+            //if repeat key frame, need refresh frame dpb
             if (Repeat_H.frame_type == KEY_FRAME)
             {
                 for (uint8_t i = 0; i < NUM_REF_FRAMES; i++)
@@ -426,14 +430,17 @@ namespace UMC_AV1_DECODER
                     if ((Repeat_H.refresh_frame_flags >> i) & 1)
                     {
                         if (!frameDPB.empty() && frameDPB[i] && frameDPB[i]->GetRefCounter())
-                           frameDPB[i]->DecrementReference();
-
+                        {
+                            AV1DecrementReference(__FUNCTION__, __LINE__, frameDPB[i]);
+                        } 
                         frameDPB[i] = const_cast<AV1DecoderFrame*>(pFrame);
-                        pFrame->IncrementReference();
+                        AV1IncrementReference(__FUNCTION__, __LINE__, pFrame);
+                       
                     }
                 }
             }
-            refs_temp = frameDPB;
+
+            last_updated_refs = frameDPB; //store updated frame_dpb
             if (pPrevFrame)
             {
                 DPBType & prevFrameDPB = pPrevFrame->frame_dpb;
@@ -601,8 +608,9 @@ namespace UMC_AV1_DECODER
         frame->SetFrameTime(frame_order * in_framerate);
         frame->SetFrameOrder(frame_order);
         FrameHeader strFrameHeader = frame->GetFrameHeader();
-        if (strFrameHeader.show_frame || strFrameHeader.show_existing_frame)//add for fix -DecoderdOrder hang issue
+        if (strFrameHeader.show_frame || frame->ShowAsExisting()) //display frame or repeat frame, frame order ++
             frame_order++;
+
     }
 
     UMC::Status AV1Decoder::GetFrame(UMC::MediaData* in, UMC::MediaData*)
@@ -615,22 +623,22 @@ namespace UMC_AV1_DECODER
 
         AV1DecoderFrame* pPrevFrame = FindFrameByUID(counter - 1);
         AV1DecoderFrame* pFrameInProgress = FindFrameInProgress();
-        DPBType updated_refs;
+        DPBType updated_refs; //store latest updated dpb
         UMC::MediaData tmper = *in;
         repeateFrame = UMC::FRAME_MID_INVALID;
 
         if ((tmper.GetDataSize() >= MINIMAL_DATA_SIZE) && pPrevFrame && !pFrameInProgress)
         {
-            if (!Repeat_show)
+            if (!last_frame_header.show_existing_frame) // if last frame is a repeat frame, don't need to call ReferenceListUpdate()
             {
-                if (!pPrevFrame->DpbUpdated())
+                if (!pPrevFrame->RefUpdated())
                 {
-                    updated_refs = DPBUpdate(pPrevFrame);
-                    refs_temp = updated_refs;
+                    updated_refs = ReferenceListUpdate(pPrevFrame);
+                    last_updated_refs = updated_refs;
                 }
                 else
                 {
-                    updated_refs = refs_temp;
+                    updated_refs = last_updated_refs;
                 }
 
             }
@@ -638,15 +646,11 @@ namespace UMC_AV1_DECODER
             {
                 DPBType const& prevFrameDPB = pPrevFrame->frame_dpb;
                 updated_refs = prevFrameDPB;
-                Repeat_show = 0;
             }
-            Curr_temp = Curr;
         }
         if (!pPrevFrame)
         {
-            updated_refs = refs_temp;
-            Curr_temp = Curr;
-            Repeat_show = 0;
+            updated_refs = last_updated_refs;
         }
 
         if ((!updated_refs.empty())&& (updated_refs[0] != nullptr))
@@ -935,6 +939,10 @@ namespace UMC_AV1_DECODER
                 }
  
                 CompleteDecodedFrames(fh, pCurrFrame, pPrevFrame);
+                if (repeatedFrame)
+                {
+                    pCurrFrame->ShowAsExisting(true);
+                }
                 CalcFrameTime(pCurrFrame);
 
                 if (!pCurrFrame)
@@ -1178,40 +1186,34 @@ namespace UMC_AV1_DECODER
         assert(size > 0);
         assert(size <= MAX_EXTERNAL_REFS);
         
-        refs_temp.resize(size);
+        last_updated_refs.resize(size);
     }
 
     void AV1Decoder::CompleteDecodedFrames(FrameHeader const& fh, AV1DecoderFrame* pCurrFrame, AV1DecoderFrame*)
     {
         std::unique_lock<std::mutex> l(guard);
-        if (Curr)
+
+        //if last frame is a repeat frame , do not insert it into output frames, its refcounter will decrease in QueryFrame()
+        if ((lastest_submitted_frame) && (!last_frame_header.show_existing_frame)) 
         {
-            FrameHeader const& FH_OutTemp = Curr->GetFrameHeader();
-            if (Repeat_show || FH_OutTemp.show_frame)
+            FrameHeader const& FH_OutTemp = lastest_submitted_frame->GetFrameHeader();
+            if (FH_OutTemp.show_frame)//display frame
             {
-                bool bAdded = false;
-                for(std::vector<AV1DecoderFrame*>::iterator iter=outputed_frames.begin(); iter!=outputed_frames.end(); iter++)
+                if (!lastest_submitted_frame->mark_Added)
                 {
-                    AV1DecoderFrame* temp = *iter;
-                    if (Curr->UID == temp->UID)
-                    {
-                        bAdded = true;
-                        break;
-                    }
+                    outputed_frames.push_back(lastest_submitted_frame);
+                    lastest_submitted_frame->mark_Added = true;
                 }
-                if (!bAdded)
-                    outputed_frames.push_back(Curr);
             }
             else
             {
-                // For no display case, decrease reference here which is increased
-                // in pFrame->IncrementReference() in show_existing_frame case.
+                // For no display frame, it decrementReference here and frame.completedecoding() in working thread
                 if(pCurrFrame)
                 {
-                    if (Curr->UID == -1)
-                        Curr = nullptr;
-                    else if(Curr != pCurrFrame)
-                        Curr->DecrementReference();
+                    if (lastest_submitted_frame->UID == -1)
+                        lastest_submitted_frame = nullptr;
+                    else if(lastest_submitted_frame != pCurrFrame)
+                        AV1DecrementReference(__FUNCTION__, __LINE__, lastest_submitted_frame);
                 }
             }
         }
@@ -1219,42 +1221,26 @@ namespace UMC_AV1_DECODER
         for(std::vector<AV1DecoderFrame*>::iterator iter=outputed_frames.begin(); iter!=outputed_frames.end(); )
         {
             AV1DecoderFrame* temp = *iter;
-            if(temp->Outputted() && temp->Displayed() && !temp->Decoded() && !temp->Repeated() && temp->DpbUpdated())
+            if(temp->Outputted() && temp->Displayed() && temp->RefUpdated())
             {
-                temp->DecrementReference();
+                AV1DecrementReference(__FUNCTION__, __LINE__, temp);
                 iter = outputed_frames.erase(iter);
             }
             else
                 iter++;
         }
 
-        // When no available buffer, don't update Curr buffer to avoid update DPB duplicated.
-        if(pCurrFrame!= NULL)
-            Curr = pCurrFrame;
-
-        if (fh.show_existing_frame)
-        {
-            Repeat_show = 1;
-        }
-        else
-        {
-            Repeat_show = 0;
-        }
+        //update lateset_submitted_frame even pCurrFrame is nullptr(not enough frame) 
+        lastest_submitted_frame = pCurrFrame;
+        last_frame_header = fh; //store latest frame header
     }
 
-    void AV1Decoder::Flush()
+    void AV1Decoder::FlushRepeatFrame(AV1DecoderFrame* frame)
     {
         std::unique_lock<std::mutex> l(guard);
-        for(std::vector<AV1DecoderFrame*>::iterator iter=outputed_frames.begin(); iter!=outputed_frames.end(); )
+        if (frame->Outputted() && frame->Displayed()) //repeat frame only need these 2 flags, as repeat frame will not call ReferenceListUpdate() function
         {
-            AV1DecoderFrame* temp = *iter;
-            if(temp->Outputted() && temp->Displayed() && !temp->Decoded() && !temp->Repeated() && temp->DpbUpdated())
-            {
-                temp->DecrementReference();
-                iter = outputed_frames.erase(iter);
-            }
-            else
-                iter++;
+            AV1DecrementReference(__FUNCTION__, __LINE__, frame);
         }
     }
 
@@ -1287,7 +1273,7 @@ namespace UMC_AV1_DECODER
         frame->Reset(&fh);
 
         // increase ref counter when we get empty frame from DPB
-        frame->IncrementReference();
+        AV1IncrementReference(__FUNCTION__, __LINE__, frame);
 
         return frame;
     }
@@ -1301,7 +1287,7 @@ namespace UMC_AV1_DECODER
 
         frame->UID = counter++;
         frame->Reset(&fh);
-        frame->IncrementReference();
+        AV1IncrementReference(__FUNCTION__, __LINE__, frame);
         return frame;
     }
 
